#!/bin/bash
#
# This script will find eosd processes which have hung
# while umounting (caused by upstream xroot issue:
# https://github.com/xrootd/xrootd/issues/418).
#
# Author: Dan van der Ster
# Date: 26 Sept 2016
#

export PATH=/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin

# make sure we have lsof
which lsof &>/dev/null || exit
 
# Find running eosd processes
PIDS=`pidof eosd`
 
# Kill the eosd processes that don't have /dev/fuse open
for PID in $PIDS
do
	if ! lsof -b -n -p $PID 2>/dev/null | grep -q /dev/fuse
	then
		# kill PID if it is still eosd. This is racey.
		logger -t eos-cleanup.sh "Killing eosd process ${PID} which doesn't have /dev/fuse open."
		grep -qs eosd /proc/${PID}/cmdline && kill -9 $PID 2>/dev/null
	fi
done

# Find mounts in /eos/* which do not have a corresponding eosd
# ONLY IF AUTOFS!!
if [ -f /etc/auto.eos ]
then
	MOUNTS=$(egrep '^eos' /proc/mounts | awk '{print $2}' | egrep '^/eos/')

	for MOUNT in $MOUNTS
	do
		EOSD=$(pgrep -u root -f "eosd ${MOUNT}")
		if [ "${EOSD}" == "" ]
		then
			logger -t eos-cleanup.sh "${MOUNT} is mounted but corresponding eosd is not running. Unmounting."
			umount -l $MOUNT
		fi
	done
fi
